import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { TokenStorageService } from './token-storage.service';
import { AppConfiguration } from 'src/app/AppConfiguration';
import { environment } from 'src/environments/environment';




const AUTH_API = environment.smartharvesterUrl + '/harvester/auth';
const FDP_URL = environment.fdpUrl;
const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' })
};


@Injectable({
  providedIn: 'root'
})
export class AuthService {



  constructor(private http: HttpClient, private tokenService: TokenStorageService, private appConfig: AppConfiguration) { }




  login(credentials): Observable<any> {
    let data: any
    data = this.http.post(AUTH_API + '/signin', {
      email: credentials.email,
      password: credentials.password
    }, httpOptions);
    return data;
  }

  register(user): Observable<any> {
    return this.http.post(AUTH_API + '/signup', {
      firstName: user.firstName,
      lastName: user.lastName,
      email: user.email,
      password: user.password
    }, httpOptions);
  }


  update(user): Observable<any> {
    return this.http.post(AUTH_API + '/user', {
      catalog_id: user.cat_id
    }, httpOptions);
  }


  getF2DSAuthToken(credentials): Observable<any> {
    return this.http.post(FDP_URL + '/tokens', {
      email: credentials.email,
      password: credentials.password
    }, httpOptions);
  }

  public isLoggedIn() {
    return this.tokenService.getToken() !== null;
  }

  public logout() {
    const config = {
      headers: new HttpHeaders({
        'Content-Type': 'application/x-www-form-urlencoded'
      })
    };
    this.tokenService.signOut();
    return this.http.post(`${AUTH_API}/logout`, null, config);
  }
}
