import { Component, OnInit } from '@angular/core';
import { FormGroup, FormControl } from '@angular/forms';
import { Router } from '@angular/router';
import { SmartHarvesterUser } from 'src/app/user/model/user';
import { AuthService } from '../services/auth.service';
import { TokenStorageService } from '../services/token-storage.service';

export interface F2DSResponse{
  token:string;
}
@Component({
  selector: 'app-signin',
  templateUrl: './signin.component.html',
  styleUrls: ['./signin.component.scss'],
})
export class SigninComponent implements OnInit {

  f2dsToken: string;
  isSubmitted = false;
  isLoggedIn = false;
  isLoginFailed = false;
  errorMessage = '';
  showPassword = false;
  user: SmartHarvesterUser = new SmartHarvesterUser({});
  

  constructor(private authService: AuthService, private router: Router, private tokenStorage: TokenStorageService) { }

  ngOnInit(): void {
    if (this.tokenStorage.getToken()) {
      // Uncomment line to avoid re-authentication if token is still valid
      //this.isLoggedIn = true;
    }
    
  }

  //get formControls() { return this.formGroup.controls; }

  onSubmit(): void {

    

    this.authService.login(this.user).subscribe(
      data => {
        this.authService.getF2DSAuthToken(this.user).subscribe((response:F2DSResponse) => {
          if (response) this.f2dsToken = response.token;
          console.log("F2ds token is : ", this.f2dsToken);
          this.tokenStorage.saveToken(data.accessToken, this.f2dsToken);
        this.tokenStorage.saveUser(data);

        this.isLoginFailed = false;
        this.isLoggedIn = true;
        console.log("User data : ", data)
        this.router.navigateByUrl('/dashboard');
        })
        
        //this.reloadPage();
      },
      err => {
        this.errorMessage = err.error.message;
        console.error("Error : ", this.errorMessage)
        this.isLoginFailed = true;
      }
    );
  }

  reloadPage(): void {
    window.location.reload();
  }


  getInputType() {
    if (this.showPassword) {
      return 'text';
    }
    return 'password';
  }

  toggleShowPassword() {
    this.showPassword = !this.showPassword;
  }

}
