import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, NavigationStart, Router } from '@angular/router';
import { NbMenuItem, NbSidebarService } from '@nebular/theme';
import { AuthService } from '../authentication/services/auth.service';
import { TokenStorageService } from '../authentication/services/token-storage.service';
import { SmartHarvesterUser } from '../user/model/user';

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent implements OnInit {

  userData: SmartHarvesterUser;
  routerUrl: string;
  menuItems: NbMenuItem[] = [
    {
      title: 'Home',
      icon: 'home-outline',
      link: '/dashboard/',
      pathMatch: 'full'
    },
    {
      title: 'Integrate your repository',
      icon: 'cube-outline',

      children: [
        {
          title: 'SDT information',
          link: '/dashboard/repositoryinfo',
          pathMatch: 'full',
          icon: 'flash-outline',
        },
        {
          title: 'Technical information',
          link: '/dashboard/repository',
          pathMatch: 'full',
          icon: 'text-outline',
        },
        {
          title: 'Describe access to datasets',
          icon: 'clipboard-outline',
          link: '/dashboard/publishapi',
          pathMatch: 'full'
        }
      ],
    },
    {
      title: 'Search',
      icon: 'search-outline',
      link: '/dashboard/simplesearch',
      pathMatch: 'full'
    },
    /*{
      title: 'Settings',
      icon: 'options-2-outline',
      children: [
        {
          title: 'FFDS settings',
          link: '/dashboard/settingfdp',
          pathMatch: 'full'
        },
        {
          title: 'SmartHarvester settings',
          link: '/dashboard/settingsmartharvester',
          pathMatch: 'full'
        }
      ],
    },*/
  ];

  constructor(private readonly sidebarService: NbSidebarService,
    private authService: AuthService,
    private tokeService: TokenStorageService, private route: Router) { }
  ngOnInit(): void {
    this.routerUrl = this.route.url;
    this.route.events.subscribe(event => {
      if (event instanceof NavigationStart) {
        this.routerUrl = event.url;
      }
    });
    let loggedIn = this.authService.isLoggedIn;
    if (loggedIn) {
      this.userData = this.tokeService.getUser();
    }

  }

  toggleSidebar(): boolean {
    this.sidebarService.toggle(true);
    return false;
  }

  logout() {
    this.authService.logout().subscribe(
      value => this.route.navigateByUrl('/auth/signin')
    );

  }
}
