import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { TokenStorageService } from 'src/app/authentication/services/token-storage.service';
import { environment } from 'src/environments/environment';

import { ParseXmlService } from '../../services/parse-xml.service';

const FDP_URL = environment.fdpUrl;
const SMARTHARVESTER_API = environment.smartharvesterUrl;
@Injectable({
  providedIn: 'root'
})
export class DatasetCrudService {

  fds2Token: string = this.sessionStorage.getFDPToken();
  smartHarvesterToken = this.sessionStorage.getToken();
  public results: string[] = [];
  itemsDataset: Object[] = [];
  ids: number[] = [];
  urls: Object[] = [];
  idsUrls: number[] = []

  constructor(
    private http: HttpClient,
    private parserService: ParseXmlService,
    private sessionStorage: TokenStorageService
  ) { }

  get requestHistoricalBaseUrl(): string {
    return environment.smartharvesterUrl + '/harvester/api/request-historical';
  }

  get tokenHarvesterHeader() {
    return {
      headers: new HttpHeaders({
        Authorization: 'Bearer ' + this.sessionStorage.getUser().token
      })
    };
  }

  async getDcatFromApi(catalogID: string, xmlUrl: string): Promise<any> {
    const httpOptions = new Headers();
    httpOptions.append('Content-Type', 'application/rdf+xml');
    httpOptions.append('Accept', 'application/rdf+xml');
    httpOptions.append('Authorization', `Bearer ${this.smartHarvesterToken}`)
    let url = `${SMARTHARVESTER_API}/harvester/api/transform?url=${xmlUrl}&catalogId=${catalogID}`;

    const myInit = { method: 'GET', headers: httpOptions, responseType: 'text' };
    const myRequest = new Request(url, myInit);

    return fetch(myRequest, myInit);
  }
  async createDatasetXml(data: string): Promise<any> {
    if (this.fds2Token) {
      const httpOptions = new Headers();
      httpOptions.append('Accept', 'application/rdf+xml');
      httpOptions.append('Content-Type', 'application/rdf+xml');
      httpOptions.append('Authorization', 'Bearer ' + this.fds2Token);


      const myInit = { method: 'POST', body: data, headers: httpOptions };
      const myRequest = new Request(`${FDP_URL}/dataset`, myInit);

      return fetch(myRequest, myInit);
    }
  }
  async createDataset(data: string): Promise<any> {
    if (this.fds2Token) {
      const httpOptions = new Headers();
      httpOptions.append('Accept', 'text/turtle');
      httpOptions.append('Content-Type', 'text/turtle');
      httpOptions.append('Authorization', 'Bearer ' + this.fds2Token);


      const myInit = { method: 'POST', body: data, headers: httpOptions };
      const myRequest = new Request(`${FDP_URL}/dataset`, myInit);

      return fetch(myRequest, myInit);
    }
  }


  draftDataset(url: string) {
    if (this.fds2Token) {
      const httpOptions = {
        headers: new HttpHeaders({
          'Content-Type': 'text/turtle',
          'Authorization': 'Bearer ' + this.fds2Token
        })
      };

      return this.http.put(`${url}/meta/state`, { current: 'PUBLISHED' }, httpOptions);
    }
  }

  updateDataset(data: string, url: string): Observable<object> {
    if (this.fds2Token) {
      const httpOptions = {
        headers: new HttpHeaders({
          'Content-Type': 'text/turtle',
          'Authorization': 'Bearer ' + this.fds2Token
        })
      };

      return this.http.put(url, data, httpOptions);
    }
  }

  createDistribution(data: string): Observable<Object> {

    if (this.fds2Token) {
      const httpOptions = {
        headers: new HttpHeaders({
          Accept: 'text/turtle',
          'Content-Type': 'text/turtle',
          'Authorization': 'Bearer ' + this.fds2Token
        })
      };

      return this.http.post(FDP_URL + '/distribution', data, httpOptions);
    }
  }

  getDatasetsFromApi(url: string, params: string): Observable<any[]> {
    const httpOptions = {
      headers: new HttpHeaders({ 'Content-Type': 'Application/json' })
    };
    return this.http.get<any[]>(`${url}/${params}`, httpOptions);
  }

  findDataSetByTitle(title: string) {

    let exist = false;
    const query1 = 'query=PREFIX dcterms: <http://purl.org/dc/terms/>  SELECT * WHERE { ?uri dcterms:title "' + escape(title) + '". }';
    this.parserService.getXmlResult(query1).subscribe(
      data => {
        if (data) {
          this.results = [];
          data.results.bindings.forEach(element => { this.results.push(element); });
          if (this.results[0]) {

            exist = true;
          }
        }
      });
    return exist;
  }

  getLocally<T = any>(path: string): Observable<T> {
    return this.http.get<T>(`${path}`);
  }
  saveDatasets(obj: Object, id: number) {
    this.itemsDataset.push(obj);
    this.ids.push(id);
  }
  resetDataset() {
    this.itemsDataset = [];
    this.ids = [];
  }

  saveUrls(url: Object, id: number) {
    this.urls.push(url);
    this.idsUrls.push(id);
  }

  resetUrls() {
    this.urls = [];
    this.idsUrls = [];
  }

  getRequestHistorical(catId: string) {
    if (this.sessionStorage.getUser().token == null) {
      return of([]);
    }

    return this.http.get<{ catId: string, request: string }[]>(this.requestHistoricalBaseUrl + '/getall/' + catId,
      this.tokenHarvesterHeader);
  }

  createRequestHistorical(catId: string, request: string) {
    if (this.sessionStorage.getUser().token == null) {
      return of();
    }

    const body = {
      catId,
      request
    };

    return this.http.post(this.requestHistoricalBaseUrl + '/create', body, this.tokenHarvesterHeader);
  }


}
