

import { HttpResponse } from '@angular/common/http';
import { ChangeDetectorRef, Component, Input, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Router } from '@angular/router';


import { Observable, of, Subscription } from 'rxjs';
import { map } from 'rxjs/operators';
import { environment } from 'src/environments/environment.prod';
import { TokenStorageService } from '../authentication/services/token-storage.service';
import { DatasetCrudService } from '../datasets/services/dataset-crud.service';

import { Property, DatasetPath, Distribution, ResponseFileTsv, OldProperty } from './class/dataset';
import { Path, RequestMapping, ResponseMapping } from './class/requestMapping';
import { FeedbackDialogComponent } from './dialog/feedback-dialog/feedback-dialog.component';
import { MappingService } from './service/mapping.service';


@Component({
  selector: 'app-mapping',
  templateUrl: './mapping.component.html',
  styleUrls: ['./mapping.component.scss']
})
export class MappingComponent implements OnInit, OnDestroy {

  addDistribution = false
  check = false;
  distributions: Property[] = [];
  ontology: string;
  itemsdataset: any[] = []
  urls: any[] = [];
  datasets: Property[];
  vocabularies: Property[];
  filteredOptions: Observable<string[]>;
  keys: string[] = [];
  selectedPaths: string[];
  distributionSelectedPaths: string[];
  datasetMappedMetadatas: Map<number, string>[] = [];
  distributionMappedMetadatas: Map<number, string>[] = [];
  index: number = 0
  first: boolean = true;
  loading = false;
  loadingCr = false;
  FDP_URL = environment.fdpUrl;
  ids: number[];
  default = true;
  @ViewChild('autoInput') input;
  @Input() type: string;
  @Input() catalogId: any;

  constructor(private ref: ChangeDetectorRef, private dataSetService: DatasetCrudService, private dialog: MatDialog, private route: Router, private tokenStorage: TokenStorageService, private mappingService: MappingService) {
    ref.detach();
    setInterval(() => {
      this.ref.detectChanges();
    }, 1000);
  }

  ngOnInit() {
    console.log(this.type)
    this.urls = this.dataSetService.urls;
    console.table(this.urls);
    this.ids = this.dataSetService.idsUrls;
    if (this.type !== "ISO 19115") {
      this.urls = this.dataSetService.urls;
      this.ids = this.dataSetService.idsUrls;
      this.itemsdataset = this.dataSetService.itemsDataset;
      console.log(this.itemsdataset)
      this.ids = this.dataSetService.ids;
      this.keys = [];
      this.type == "Dataverse" ? this.getKeysFromMetadataCustom(this.itemsdataset[0], '') : this.getKeysFromMetadataCustom(this.itemsdataset[0], '');
      this.keys = Array.from(new Set(this.keys));
      this.filteredOptions = of(this.keys);
    }

  }



  selectOntology(): void {
    this.dataSetService.getLocally(`./assets/geodcat.json`).subscribe(
      datasets => {
        this.datasets = this.ontology === "dcat" ? datasets.filter(d => !d.geodcat) : datasets;
        this.vocabularies = this.ontology === "dcat" ? datasets.filter(d => !d.geodcat) : datasets;
        this.vocabularies.forEach(dataset => dataset.dcatClass = 'dcat:dataset');
        this.selectedPaths = [];
        this.dataSetService.getLocally('./assets/distribution.json').subscribe(
          (distributions: Distribution[]) => {
            this.distributions = this.ontology === "dcat" ? distributions.filter(d => !d.geodcat) : distributions;
            this.distributions.forEach(distribution => distribution.dcatClass = 'dcat:distribution');
            this.vocabularies = this.vocabularies.concat(this.ontology === "dcat" ? this.distributions.filter(d => !d.geodcat) : distributions);
            this.distributionSelectedPaths = [];
          }
        );
      },
      error => {
        console.error(error);
      },
    );

  }


  mapFromXslt() {
    this.loading = true;
    this.itemsdataset = [];
    let requestPromises: Promise<any>[] = [];
    this.urls.forEach((url: string) => {
      let requestPromise: Promise<void>;

      requestPromise = this.dataSetService.getDcatFromApi(this.catalogId, url)
        .then((response: Response) => {
          response.text()
            .then((text) => {
              this.itemsdataset.push(text);
            });
        });

    });
    Promise.all(requestPromises).finally(() => this.loading = false);
  }


  createDataset(item: Object, classDcat: string): Map<number, string> {
    let mappedMetadata: Map<number, string> = new Map()
    const array: string[] = (classDcat === "dataset") ? this.selectedPaths : this.distributionSelectedPaths
    for (let i = 0; i < array.length; i++) {
      if (array[i] && array.length > 0) {
        let tab = array[i].split(' : ');
        this.type == "Dataverse" ? mappedMetadata.set(i, this.getValueCustom(tab, item)) : mappedMetadata.set(i, this.getValueCustom(tab, item));
      }
    }
    this.loadingCr = false;
    return mappedMetadata;
  }

  publishDataset(): void {
    let paths: Path[] = [];

    for (let i = 0; i < this.datasets.length; i++) {
      if (this.selectedPaths[i]) {
        paths.push(new Path(this.datasets[i].uri, this.selectedPaths[i], this.datasets[i].card, this.datasets[i].dcatClass));
      }
    }
    for (let i = 0; i < this.distributions.length; i++) {
      if (this.distributionSelectedPaths[i]) {
        paths.push(new Path(this.distributions[i].uri, this.distributionSelectedPaths[i], this.distributions[i].card, this.distributions[i].dcatClass));
      }
    }


    let data: RequestMapping = new RequestMapping(this.urls, paths, this.tokenStorage.getFDPToken());

    let postedDatasets = [];
    let notPostedDatasets = [];
    this.loading = true;
    const requestPromises: Promise<any>[] = [];

    let requestPromise = this.mappingService.postToFdpFropSmartharvester(this.catalogId, data).then( resp => {
      if (resp) {
        console.log(resp);
        resp.json().then((data: any) => {
          console.log(data.message);
          data.publishedUrl.forEach(e => postedDatasets.push(e));
          data.notPublishedUrl.forEach(e => notPostedDatasets.push(e));
        });
      }
    
      /*const location = resp.headers.get("location");
      const datasetId = location.substring(location.search("dataset/") + 8, location.length);*/

    });
    requestPromises.push(requestPromise);

    Promise.all(requestPromises).finally(() => {
      this.loading = false;
      this.dialog.open(FeedbackDialogComponent, {
        data: {
          postedMetadatas: postedDatasets,
          notPostedMetadatas: notPostedDatasets
        }
      }).afterClosed().subscribe();

    })
  }

  buildRDF(vocabularies: Property[], array: Map<number, string>[], i: number): string {
    let properties = "";
    let version = false;
    array[i].forEach((value: string, key: number) => {
      let uri = vocabularies[key].uri;
      if (uri === 'dct:hasVersion') {
        version = true;
      }

      if (Array.isArray(value)) {
        if (this.isReplicable(this.getCard(uri))) {
          value.forEach(v => {
            properties += this.writeRdf(uri, v);
          })
        } else {
          properties += this.writeRdf(uri, value[0]);
        }
      } else {
        properties += this.writeRdf(uri, value);
      }

    })

    if (!version) properties += `dct:hasVersion "undefined";\n`;

    return properties;
  }

  publishDatasetFromGeodcatApi() {
    let postedDatastes = [];
    let notPostedDatasets = [];
    this.loading = true;
    const requestPromises: Promise<any>[] = [];
    for (let i = 0; i < this.urls.length; i++) {

      let requestPromise = this.dataSetService.createDatasetXml(this.urls[i]).then((resp: HttpResponse<any>) => {
        if (resp.status.toString().startsWith('2')) {
          postedDatastes.push(this.ids[i]);
        } else {
          notPostedDatasets.push(this.ids[i]);
        }
      })
      requestPromises.push(requestPromise);
    }
    Promise.all(requestPromises).finally(() => {
      this.loading = false;
      this.dialog.open(FeedbackDialogComponent, {
        data: {
          postedMetadatas: postedDatastes,
          notPostedMetadatas: notPostedDatasets
        }
      }).afterClosed().subscribe();

    })
  }




  private writeRdf(key: string, value: string): string {
    let properties = "";
    value = (value && typeof value === "string") ? value.replace(/\n/g, '') : value;

    switch (key) {
      case 'dcat:landingPage':
      case 'dcat:theme':
      case 'dcat:contactPoint':
      case 'dcat:accessURL':
      case 'dcat:downloadURL':
      case 'dct:language':
        properties += key + ' <' + value + '>;\n';
        break;
      case 'dct:issued':
      case 'dct:modified':
        properties += key + ' "' + value + '"^^xsd:dateTime;\n';
        break;
      default:
        properties += key + ' "' + value + '";\n'; break;
    }
    return properties;
  }


  /* function to get recursively all the keys and values from the JSON object */

  getKeysFromMetadataCustom(obj: Object, keyParent: string) {
    Object.keys(obj).forEach(key => {
      if (typeof obj[key] === 'object') {
        if (Array.isArray(obj[key])) {
          obj[key].forEach(e => {
            if (typeof e === 'object') {
              Object.keys(e).forEach(k => {
                this.keys.push(keyParent + k);
              })
              this.getKeysFromMetadataCustom(e, keyParent + key + ' : ');
            }
          });
          this.keys.push(keyParent + key)
          this.getKeysFromMetadataCustom(obj[key], keyParent + key + ' : ');
        } else {
          this.keys.push(keyParent + key)
          this.getKeysFromMetadataCustom(obj[key], keyParent + key + ' : ');
        }
      } else {

        this.keys.push(keyParent + key);

      }
    });
  }


  getKeysFromMetadataDataverse(obj: Object, keyParent: string) {

    Object.keys(obj).forEach(key => {
      if (typeof obj[key] === 'object') {
        if (Array.isArray(obj[key])) {
          if (keyParent) {
            this.keys.push(keyParent + ' : ' + key)
          } else {
            this.keys.push(key)
          }
          obj[key].forEach(e => {
            if (typeof e === 'object') {
              this.getKeysFromMetadataDataverse(e, keyParent + ' : ' + key + ' : ' + obj[key].indexOf(e));
            } else {
              this.keys.push(keyParent + ' : ' + key + ' : ' + obj[key].indexOf(e))
            }
          });
        } else {
          if (keyParent) {
            this.getKeysFromMetadataDataverse(obj[key], keyParent + ' : ' + key);
          } else {
            this.getKeysFromMetadataDataverse(obj[key], key);
          }
        }
      } else {
        if (keyParent) {
          this.keys.push(keyParent + ' : ' + key)
        } else {
          this.keys.push(key)
        }
      }
    });
  }

  private getValueCustom(tab: string[], item: Object) {
    let obj: Object;
    if (tab.length == 1) {
      return item[tab[0]];
    } else {
      for (let i = 0; i < tab.length; i++) {
        if (i == 0) {
          obj = item[tab[i]]
        } else if (i < tab.length - 1 && obj[tab[i]]) {
          obj = obj[tab[i]];
        } else {
          if (Array.isArray(obj) && obj.some(e => typeof e === 'object')) {
            let array = [];
            obj.forEach(e => {
              if (e[tab[i]]) {
                array.push(e[tab[i]]);
              }
            });
            return array;
          } else if (typeof obj === 'object' && Object.values(obj).every(e => Array.isArray(e))) {
            let array = [];
            for (let element in obj) {
              obj[element].forEach(e => {
                if (e[tab[i]]) {
                  array.push(e[tab[i]]);
                }
              })
            }
            return array;
          } else if (Array.isArray(obj[tab[i]])) {
            return Object.values(obj[tab[i]]);
          } else {
            return obj[tab[i]];
          }
        }
      }
    }

  }

  private getValueDataverse(tab: string[], item: Object): string {
    let obj: Object
    for (let i = 0; i < tab.length; i++) {
      if (tab.length == 1) {
        return item[tab[0]];
      } else {
        if (i == 0) {
          obj = item[tab[i]]
        } else if (i < tab.length - 1 && obj[tab[i]]) {
          obj = obj[tab[i]];
        } else {
          if (obj[tab[i]] && typeof obj[tab[i]] !== 'object') {
            return obj[tab[i]];
          } else {
            if (i == tab.length - 2) {
              this.getKeysFromMetadataDataverse(item, "");
              console.log(tab[i] + "  " + i)
              this.keys.forEach(k => {
                let table = k.split(" : ");
                if (table[table.length - 2] === tab[i]) {
                  for (let i = 0; i < tab.length; i++) {
                    if (table.length == 1) {
                      return item[tab[0]];
                    } else {
                      if (i == 0) {
                        obj = item[table[i]]
                      } else if (i < table.length - 1 && obj[table[i]]) {
                        obj = obj[table[i]];
                      } else {
                        if (obj[table[i]] && typeof obj[tab[i]] !== 'object') {
                          return obj[table[i]];
                        } else {
                          return 'undefined';
                        }
                      }
                    }
                  }
                }
              })
            }
          }
        }
      }
    }
  }

  isArray(item: any): boolean {
    return Array.isArray(item);
  }

  mapDataset() {
    this.loadingCr = true;
    this.datasetMappedMetadatas = [];
    this.distributionMappedMetadatas = [];
    this.itemsdataset.forEach((dataset: Object) => {
      this.datasetMappedMetadatas.push(this.createDataset(dataset, "dataset"));
      this.distributionMappedMetadatas.push(this.createDataset(dataset, "distribution"))
    })
    console.table(this.datasetMappedMetadatas);
    console.log(this.distributionMappedMetadatas);
    this.first = false;
  }
  next() {
    //this.loadingCr = true;
    if (this.index < this.itemsdataset.length) {
      this.index += 1;
      //this.createDataset(this.itemsdataset[this.index])
      console.log('index = ' + this.index);
    }
  }
  prev() {
    //this.loadingCr = true;
    if (this.index > 0) {
      this.index -= 1;
    }
    //this.createDataset(this.itemsdataset[this.index])
    console.log('index = ' + this.index);
  }

  createObjectToExport(): DatasetPath[] {
    let datasetPathArray: DatasetPath[] = []
    for (let i = 0; i < this.datasets.length; i++) {
      if (this.selectedPaths[i]) {
        datasetPathArray.push(new DatasetPath(this.datasets[i], this.selectedPaths[i], 'dcat:dataset', this.datasets[i].card));
      }
    }
    for (let i = 0; i < this.distributions.length; i++) {
      if (this.distributionSelectedPaths[i]) {
        datasetPathArray.push(new DatasetPath(this.distributions[i], this.distributionSelectedPaths[i], 'dcat:distribution', this.distributions[i].card))
      }
    }
    return datasetPathArray;
  }

  downloadJson() {

    let datasetPathArray = this.createObjectToExport();




    this.mappingService.downloadFile(datasetPathArray);
  }

  openFileInputDialog(): void {
    document.getElementById('file').click();
  }

  fillFromTsv(vocabulary: Property[], paths: string[], tsvFile: ResponseFileTsv[], dcatClass: string) {
    for (let i = 0; i < tsvFile.length; i++) {
      if (i > 0 && tsvFile[i].object_id === tsvFile[i - 1].object_id) {
        const voc: Property = vocabulary.find(e => e.uri === tsvFile[i].object_id);
        const index = vocabulary.indexOf(voc);
        vocabulary.splice(index + 1, 0, voc);
        paths.splice(index + 1, 0, tsvFile[i].subject_label);
      } else {
        const voc: Property = vocabulary.find(e => e.uri === tsvFile[i].object_id);
        const index = vocabulary.indexOf(voc);
        paths[index] = tsvFile[i].subject_label;
      }
    }
  }


  importJson(event: Event) {
    const files: FileList = (event.target as HTMLInputElement).files;
    const jsonFile = files[0];
    const reader = new FileReader();

    reader.onloadend = () => {
      if (jsonFile.name.split('.')[1] === "tsv") {
        let paths: ResponseFileTsv[] = <ResponseFileTsv[]>this.mappingService.tsvToJson(reader.result as string);
        let datasetPath: ResponseFileTsv[] = [];
        let distributionPath: ResponseFileTsv[] = [];
        console.table(paths)
        paths.forEach(path => {
          (path.object_category.includes('dataset')) ? datasetPath.push(path) : distributionPath.push(path);
        })

        console.log(datasetPath)
        console.log(distributionPath)
        if (distributionPath.length > 0) this.addDistribution = true;
        this.fillFromTsv(this.datasets, this.selectedPaths, datasetPath, 'dcat:dataset');
        this.fillFromTsv(this.distributions, this.distributionSelectedPaths, distributionPath, 'dcat:distribution');
      } else {
        let paths: DatasetPath[] = <DatasetPath[]>JSON.parse(reader.result as string);
        console.table(paths)

        for (let i = 0; i < paths.length; i++) {
          if (i > 0 && paths[i].dcatProperty === paths[i - 1].dcatProperty) {
            const voc: Property = this.datasets.find(e => e.uri === (<Property>paths[i].dcatProperty).uri);
            this.datasets.splice(i + 1, 0, voc);
            this.selectedPaths.splice(i + 1, 0, paths[i].path);

            if (typeof paths[i].dcatProperty !== 'object') {
              this.datasets[i] = this.vocabularies.filter(e => e.dcatClass === 'dcat:dataset').find(v => v.uri === <string>paths[i].dcatProperty);
            } else {
              this.datasets[i] = this.vocabularies.filter(e => e.dcatClass === 'dcat:dataset').find(v => v.uri === (<OldProperty><unknown>paths[i].dcatProperty).identifier);
            }

          } else {
            this.selectedPaths[i] = paths[i].path;
            if (typeof paths[i].dcatProperty !== 'object') {
              this.datasets[i] = this.vocabularies.find(v => v.uri === <string>paths[i].dcatProperty);
            } else {
              this.datasets[i] = this.vocabularies.find(v => v.uri === (<OldProperty><unknown>paths[i].dcatProperty).identifier);
            }
          }


        }
      }
    }
    reader.readAsText(jsonFile);
    this.ref.markForCheck()


  }

  addField(index: number, properties: Property[], paths: string[]) {
    let addedDcat: Property = properties.find(e => e.uri === properties[index].uri);
    properties.splice(index + 1, 0, addedDcat);

    paths.splice(index + 1, 0, '');

   
  }

  deleteField(index: number, properties: Property[], paths: string[]) {
    properties.splice(index, 1);
    paths.splice(index, 1);
  }

  // to delete a property of dcat dataset mapped
  deleteProperty(key: number, map: Map<number, string>[]) {
    map[this.index].delete(key);
  }

  trackByIndex(index: number): any {
    return index;
  }

  /* autocompletion functions */

  private filter(value: string): string[] {
    let filterValue = value.toLowerCase();
    return this.keys.filter(optionValue => optionValue.toLowerCase().includes(filterValue));
  }

  getFilteredOptions(value: string): Observable<string[]> {
    return of(value).pipe(
      map(filterString => this.filter(filterString)),
    );
  }
  onModelChange(value: string) {
    this.filteredOptions = of(this.filter(value));
  }
  reset() {
    this.filteredOptions = of(this.keys);
  }

  toggle(checked: boolean) {
    this.default = checked;
  }

  toggleDistri(checked: boolean) {
    this.addDistribution = checked;
  }

  isMandatory(card: string): boolean {
    return card.startsWith("1");
  }

  isReplicable(card: string): boolean {
    return card.endsWith("n");
  }

  getCard(uri: string): string {
    return this.vocabularies.find((vocabulary: Property) => vocabulary.uri === uri).card;
  }

  ngOnDestroy(): void {
    let datasetPathArray = this.createObjectToExport();
    this.mappingService.postFile(datasetPathArray, this.catalogId).subscribe();
  }
  private getDistributionEmpty(properties: string, datasetId: string): string {
    return `@prefix dcat: <http://www.w3.org/ns/dcat#>.
    @prefix dct: <http://purl.org/dc/terms/>.
    @prefix adms: <http://www.w3.org/ns/adms#> .
    @prefix dqv:  <http://www.w3.org/ns/dqv#> .
    @prefix geodcat: <http://data.europa.eu/930/>.
    @prefix prov: <http://www.w3.org/ns/prov#>. 
    @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>. 
    @prefix skos: <http://www.w3.org/2004/02/skos/core#>. 
    @prefix spdx:  <http://spdx.org/rdf/terms#>. 
    @prefix foaf: <http://xmlns.com/foaf/0.1/>. 
    @prefix odrl: <http://www.w3.org/ns/odrl/2/>. 
    @prefix cnt:   <http://www.w3.org/2011/content#>.
    @prefix language: <http://id.loc.gov/vocabulary/iso639-1/>.
    @prefix s: <${this.FDP_URL}/>.
    @prefix c: <${this.FDP_URL}/dataset/>.
    
    s:new
    a dcat:Distribution, dcat:Resource;
    dct:isPartOf c:${datasetId};
    ${properties}.`;
  }

  private getDatasetEmpty(properties: string): string {
    return `@prefix dcat: <http://www.w3.org/ns/dcat#>.
    @prefix dct: <http://purl.org/dc/terms/>.
    @prefix adms: <http://www.w3.org/ns/adms#> .
    @prefix dqv:  <http://www.w3.org/ns/dqv#> .
    @prefix geodcat: <http://data.europa.eu/930/>.
    @prefix prov: <http://www.w3.org/ns/prov#>. 
    @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>. 
    @prefix language: <http://id.loc.gov/vocabulary/iso639-1/>.
    @prefix s: <${this.FDP_URL}/>.
    @prefix c: <${this.FDP_URL}/catalog/>.
    
    s:new
    a dcat:Dataset, dcat:Resource;
    dct:isPartOf c:${this.catalogId} ;
    ${properties}.`;
  }
}


