import { HttpClient, HttpHeaders, HttpRequest } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { TokenStorageService } from 'src/app/authentication/services/token-storage.service';
import { SmartHarvesterUser } from 'src/app/user/model/user';
import { environment } from 'src/environments/environment';
import { DatasetPath } from '../class/dataset';
import { RequestMapping } from '../class/requestMapping';


@Injectable({
  providedIn: 'root'
})

export class MappingService {

  smartHarvesterUrl = environment.smartharvesterUrl;
  fdpUrl = environment.fdpUrl;
  fds2Token: string = this.storageService.getToken();

  constructor(private storageService: TokenStorageService, private http: HttpClient) { }

  postFile(objArray: DatasetPath[], catalogId: String) {

    



    const body: Blob = new Blob(['\ufeff' + this.convertToTSV(objArray, [
      'subject_label',
      'predicate_id',
      'object_id',
      'match_type',
      'object_category'
    ])], {
      type: 'text/tsv;charset=utf-8;',
    });

    const data: FormData = new FormData();
    data.append('file', body);
    const req = new HttpRequest('POST', `${this.smartHarvesterUrl}/harvester/api/storeMapping/upload/${catalogId}`, data, {
      headers: new HttpHeaders({
        
        'Authorization': 'Bearer ' + this.fds2Token
      }),
		  reportProgress: true,
		  responseType: 'json'
		});
	
		return this.http.request(req);


  }

  downloadFile(data: DatasetPath[], filename = 'dcatMapping') {
    let tsvData = this.convertToTSV(data, [
      'subject_label',
      'predicate_id',
      'object_id',
      'match_type',
      'object_category'
    ]);

    let blob = new Blob(['\ufeff' + tsvData], {
      type: 'text/tsv;charset=utf-8;',
    });
    let dwldLink = document.createElement('a');
    let url = URL.createObjectURL(blob);
    let safariBrowser = navigator.userAgent.indexOf('Safari') != -1 && navigator.userAgent.indexOf('Chrome') == -1;
    if (safariBrowser) {
      dwldLink.setAttribute('target', '_blank');
    }
    dwldLink.setAttribute('href', url);
    dwldLink.setAttribute('download', filename + '.tsv');
    dwldLink.style.visibility = 'hidden';
    document.body.appendChild(dwldLink);
    dwldLink.click();
    document.body.removeChild(dwldLink);
  }

  convertToTSV(objArray: DatasetPath[], headerList: string[]): string {
    let date = new Date();
    let user: SmartHarvesterUser = this.storageService.getUser();
    let array = typeof objArray != 'object' ? JSON.parse(objArray) : objArray;
    let str = `#mapping_date: ${date.getDate()}/${date.getMonth()}/${date.getFullYear()}\n#creator_label: ${user.firstName} ${user.lastName}\n#curie_map:\n#  dcat: "http://www.w3.org/ns/dcat#"\n#  dct: "http://purl.org/dc/terms/"\n#  adms: "http://www.w3.org/ns/adms#\n#  dqv: "http://www.w3.org/ns/dqv#"\n#  geodcat: "http://data.europa.eu/930/"\n#  prov: "http://www.w3.org/ns/prov#"\n# rdfs: "http://www.w3.org/2000/01/rdf-schema#"`;
    let row = '';

    for (let index in headerList) {
      row += headerList[index] + '\t';
    }
    row = row.slice(0, -1);
    str += row + '\n';
    for (let i = 0; i < array.length; i++) {

      if (array[i].path) {
        let line = '';
        for (let index in headerList) {
          let head = headerList[index];
          switch (head) {
            case 'subject_label':
              line += array[i].path + '\t';
              break;
            case 'predicate_id':
              line += 'skos:exactMatch' + '\t';
              break;
            case 'object_id':
              line += array[i].dcatProperty.uri + '\t';
              break;
            case 'match_type':
              line += 'lexical' + '\t';
              break;
            case 'object_category':
              line += array[i].dcatClass + '\t';
          }
        }
        str += (i != array.length - 1) ? line + '\n' : line;
      }



    }
    return str;
  }

  tsvToJson(tsv: string): Object {
    let lines: string[] = tsv.split('\n');
    let result = [];
    console.table(lines)
    while (lines[0].startsWith('#')) {
      lines.shift();
    }

    console.table(lines)
    let headers = lines[0].split('\t');
    lines.shift()
    for (let i = 0; i < lines.length; i++) {

      let obj = {};
      let currentLine = lines[i].split('\t');

      for (var j = 0; j < headers.length; j++) {
        obj[headers[j]] = currentLine[j];
      }

      result.push(obj);
    }

    return result;
  }

  async postToFdpFropSmartharvester(catalogId: string, data: RequestMapping): Promise<any> {
    
    if(this.fds2Token) {
    const httpOptions = new Headers();
    httpOptions.append('Content-Type', 'application/json');
    httpOptions.append('Accept', 'application/json');
    httpOptions.append('Authorization', `Bearer ${this.fds2Token}`)
    let url = `${this.smartHarvesterUrl}/harvester/api/transform/publish?fdpUrl=${this.fdpUrl}&catalogId=${catalogId}`;

    const myInit = { method: 'POST', body: JSON.stringify(data), headers: httpOptions};
    const myRequest = new Request(url, myInit);

    return fetch(myRequest, myInit);
    }

  }
}
