import HttpStatusCode, { HttpMethod } from './http-enum'
import { ParameterType, ShemaType } from './openapi-enum';

export enum OpenApiTag {
    search = 'search',
    dataset = 'dataset',
    other = 'other'
}

export interface ServerDTO {
    url: string;
}

export interface SchemaDTO {
    type: ShemaType,
    default: string
}

export interface ParameterDTO {
    in: ParameterType,
    name: string,
    schema: SchemaDTO,
    required: boolean
}

export interface ItemDTO {
    type: string
}
export interface ResponseSchemaDTO {
    type: string,
    items: ItemDTO
}

export interface ContentDTO {
    example: string;
    contentType: string;
}

export type MimeTypeToContentMapDTO = Map<string, ContentDTO>;

export interface ResponseDTO {
    description: string;
    content: any;
    httpStatusCode: string;
}

export type HttpCodeToResponseMapDTO = Map<HttpStatusCode, ResponseDTO>;

export interface RequestDTO {
    httpmethod: string;
    title: string;
    summary: string;
    tags: string[];
    parameters: ParameterDTO[];
    responses: any;
}

export type MethodToRequestMapDTO = Map<HttpMethod, RequestDTO>;

export type PathMapDTO = Map<string, MethodToRequestMapDTO>;

export type InfoMapDTO = {
    title: string;
    version: string;
    description: string;
    'x-catalog-id': string;
    'x-format': string;
    'x-page-param': string;
    'x-result': string;
    'x-start-param': string;
};

export interface OpenApiDTO {
    openapi: string;
    info: InfoMapDTO;
    servers: ServerDTO[];
    paths: any;
}

