import HttpStatusCode, { HttpMethod } from './http-enum'
import { ServerDTO } from './openapi-dto';
import { ParameterType } from './openapi-enum';

export interface Server {
    url: string;
}

export interface Schema {
    type: string,
    default: string
}

export interface Parameter {
    in: ParameterType;
    name: string;
    schema: Schema;
    required: boolean;
    description: string;
}

export interface Item {
    type: string
}

export interface ResponseSchema {
    type: string,
    items: Item
}

export interface Content {
    contentType: string;
    example: string;
}

export interface Response {
    httpStatusCode: HttpStatusCode;
    description: string;
    contents: Content[];
}

export interface Request {
    title: string,
    summary: string,
    tags: string[]
    parameters: Parameter[],
    responses: Response[],
    httpmethod: HttpMethod,
    toggled: boolean
}

export interface Path {
    pathName: string,
    requests: Request[],
}

export class InfoMap {
    title: string;
    version: string;
    description: string;
    'x-catalog-id': string;
    'x-format': string;
    'x-page-param': string;
    'x-result': string;
    'x-start-param': string;

    constructor(catId: string) {
        this['x-catalog-id'] = catId;
    }
};

export class OpenApi {
    openapi = '3.0.0';
    info: InfoMap;
    private _servers: Server[];
    paths: Path[];

    get servers() {
        return this._servers;
    }

    constructor(catId: string, server: string) {
        this.info = new InfoMap(catId);
        this._servers = [{ url: server }];
        this.paths = [];
    }

    cleanServers(serverUrl: string) {
        if (serverUrl == null || serverUrl.length === 0) {
            return;
        }

        this._servers = [{ url: serverUrl }];
    }

    addServers(serverUrl: string) {
        if (serverUrl == null || serverUrl.length === 0) {
            return;
        }

        for (const server of this._servers) {
            if (server.url === serverUrl) {
                return;
            }
        }
        this._servers.push({ url: serverUrl });
    }
}

