import { Component, OnInit } from '@angular/core';
import { Validators } from '@angular/forms';
import { FileSaverService } from 'ngx-filesaver';
import { FormControl } from '@angular/forms';
import { FormGroup } from '@angular/forms';
import { environment } from 'src/environments/environment.prod';
import { PublishRepositoryService } from './services/publish-repository.service';
import { NbGlobalPhysicalPosition, NbToastrService } from '@nebular/theme';
import { Router } from '@angular/router';


@Component({
  selector: 'app-repository',
  templateUrl: './repository.component.html',
  styleUrls: ['./repository.component.scss']
})

export class RepositoryComponent implements OnInit {

  public fileName: string;
  public importFile: File;
  public resultat: any;
  public persistentUrl: string;
  FDP_URL = environment.fdpUrl;

  Form = new FormGroup({
    repotype: new FormControl(),
    reponame: new FormControl('', Validators.required),
    repodescription: new FormControl(),
    repourl: new FormControl(),
    repoversion: new FormControl(),
    repolicence: new FormControl(),
    repolanguage: new FormControl(),
    filetofill: new FormControl(),
  });


  constructor(
    private _FileSaverService: FileSaverService,
    private publishService: PublishRepositoryService,
    private toastrService: NbToastrService,
    private router: Router
  ) { }

  ngOnInit() {

    this.publishService.getPersistentUrl().subscribe({
      next: (response) => this.persistentUrl = response.persistentUrl
    });
  }

  yamlToRepository() {
    const files = (event.target as HTMLInputElement).files;
    this.importFile = files[0];

    let text = '';
    let lines = [];
    const map = new Map<string, string>();
    const fileReader = new FileReader();


    fileReader.onload = () => {
      text = fileReader.result as string;

      lines = text.split('\n');
      for (const line of lines) {
        const parameter: string[] = line.split(': ');

        if (parameter == null || parameter.length !== 2) {
          continue;
        }

        map.set(parameter[0].trim(), parameter[1]);
      }

      this.Form.setValue({
        repotype: this.formatValue(map.get('type')),
        reponame: this.formatValue(map.get('title')),
        repodescription: this.formatValue(map.get('description')),
        repourl: this.formatValue(map.get('url')),
        repoversion: this.formatValue(map.get('version')),
        repolicence: this.formatValue(map.get('licence')),
        repolanguage: this.formatValue(map.get('language')),
        filetofill: ''
      });
    };
    fileReader.readAsText(this.importFile);
  }

  private formatValue(value: string): string {
    if (value == null) {
      return '';
    }

    value = value.trim();

    if (value === 'null') {
      return '';
    }

    return value;
  }

  repositorytoyaml() {
    let data: string;


    data = '\
repository:\n\
  type: '+ this.Form.value.repotype + '\n\
  title: '+ this.Form.value.reponame + '\n\
  description: '+ this.Form.value.repodescription + '\n\
  url: '+ this.Form.value.repourl + '\n\
  version: '+ this.Form.value.repoversion + ' \n\
  licence: '+ this.Form.value.repolicence + '\n\
  language: '+ this.Form.value.repolanguage + '\n\
'

    const fileName = `repository.yaml`;
    const fileType = this._FileSaverService.genType(fileName);
    const txtBlob = new Blob([data], { type: fileType });

    this._FileSaverService.save(txtBlob, fileName);
  }


  publishCatalog() {
    let data: string;

    data = '\
@prefix dcat: <http://www.w3.org/ns/dcat#>.\n\
@prefix dct: <http://purl.org/dc/terms/>.\n\
@prefix foaf:  <http://xmlns.com/foaf/0.1/> .\n\
@prefix lang: <http://id.loc.gov/vocabulary/iso639-1/>. \n\
@prefix fdp: <'+ this.FDP_URL + '/>.\n\nfdp:new \n\
    a dcat:Catalog, dcat:Resource;\n\
    dct:description "'+ this.Form.value.repodescription + '";\n\
    dct:hasVersion "'+ this.Form.value.repoversion + '";\n\
    dct:isPartOf <' + this.persistentUrl + '>;\n\
    dcat:keyword "'+ this.Form.value.repotype + '";\n\
    dct:title "'+ this.Form.value.reponame + '";\n\
    foaf:homepage <' + this.Form.value.repourl + '>;\n\
    dct:licence <' + this.Form.value.repolicence + '>;\n\
    dct:language lang:' + this.Form.value.repolanguage + ' .\n';

    this.publishService.publishRepository(data).then(() => {
      this.toastrService.show('Created', 'Catalog created !',
        { position: NbGlobalPhysicalPosition.TOP_RIGHT, status: 'success' });
      this.router.navigateByUrl('/dashboard/publishapi');
    }).catch(() => {
      this.toastrService.show('Error', 'Catalog created !',
        { position: NbGlobalPhysicalPosition.TOP_RIGHT, status: 'warning' });
    });


  }
}
