import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { TokenStorageService } from '../authentication/services/token-storage.service';

export interface FdpApiResponseItem {
  subject: {
    namespace: string,
    localName: string
  };
  predicate: {
    namespace: string,
    localName: string
  };
  object: any;
  context: string;
}

const FDP_URL = environment.fdpUrl;

@Injectable({
  providedIn: 'root'
})
export class CatalogService {

  httpOptionsSmartHarvester = {
    headers: new HttpHeaders({
      Authorization: 'Bearer ' + this.tokenService.getToken()
    })
  };
  httpOptionsFDP = {
    headers: new HttpHeaders({
      Authorization: 'Bearer ' + this.tokenService.getFDPToken()
    })
  };

  constructor(
    private tokenService: TokenStorageService,
    private http: HttpClient
  ) { }

  get smartApiUrl(): string {
    return environment.smartharvesterUrl + '/harvester/api';
  }

  getAllCatalogId() {
    const user = this.tokenService.getUser();
    return this.http.get<{ catId: string, uuid: string }[]>(this.smartApiUrl + '/user/' + user.email + '/catalogs',
      this.httpOptionsSmartHarvester);
  }

  getCatalogContentById(catId: string) {
    return this.http.get<FdpApiResponseItem[]>(FDP_URL + '/catalog/' + catId, this.httpOptionsFDP);
  }

  deleteCatalog(catId: string) {
    return this.http.delete(this.smartApiUrl + '/catalogs/' + catId, this.httpOptionsSmartHarvester);
  }
}
